# Command Line Interfaces

Command line interfaces (CLI) are a common way to efficiently interact with a computer. While we are largely familiar with GUIs (graphical user interfaces) these days, many powerful computer tools exist without a GUI to control it. These applications are run and controlled through a CLI.



## How to read these worksheets!

These worksheets are written using markdown, which is a common way for communicating for modern developers. Importantly it allows us to insert code blocks ...

```
like so.

Notice in here that the font is monospaced ... like code.
```



In these worksheets, you may find code in these blocks, but we will also use it to represent our CLI.



In these cases, commands are written per line. To run a command in a CLI, type the command and then press enter.

Usually the out put will be omitted, however, if we want to represent some output from the CLI, we may indicate this with a ```>``` character at the start. ***Don't type this in***, it isn't a command...

Also comments are lines that begin with ```#``` and will usually be highlighted differently. Again don't type these - imagine them as the voice of me, the narrator.

```bash
bash command

> This line represents the output of a command being run

# This is a comment, here to provide guidance and not to be written in the console

# example of running git with the 'init' command

git init

> Initialized empty Git repository in /path/to/new/repo
```



## What CLI can I use?


#### Windows

There are a few options for CLI for windows, for this module, we recommend use ```power shell ```. Powershell is the name of the application, as well as, the name of the shell command language. Follow the section 'A quick tour of Powershell` to get a feel for it. We recommend you also try some of the further reading/watching in the resources section.

To run powershell you have two options:

**From the Start Menu**

1. Click Start, type **PowerShell** and then click **Windows PowerShell**.
2. From the Start menu, click Start, click All Programs, click Accessories, click the **Windows PowerShell** folder, and then click **Windows PowerShell**.

> You can also use bash on windows if prefered by installing the Ubuntu Linux subsystem. This does take up a little space but is an excellent utility to have on windows!

> You can optionally use windows CMD, however we will not be providing support for this on this module, so this would require independent learning. If you are allready familiar with CMD, please feel free to use it.

#### Linux & OS X 

On Unix platforms we will use the default terminal. This uses bash as the shell language and so you shoudl follow the section 'A Quick Tour of Bash'.

This can be launched on...

**Ubuntu**: ```ctrl``` + ```alt``` + ```t```

**Gnome**: ```alt``` + ```f2```

**OS X** : ```cmd``` + ```space``` (to launch spotlight), then type ***'terminal'*** + hit ```enter```



## Terminology

The following terms can often be considered interchangeable for describing command line interfaces. While in some cases they do mean specific things - in conversation people often use them to refer to the CLI

- console

- Terminal 

- Command prompt / prompt

- Shell 

  

Some more specific definitions:

**Bash** is a Unix shell and command language.

**PowerShell** is a Windows shell and command language that is the successor to **cmd** (batch).

A **shell** is a interface that provides access to an operating system's services. 

A [**command**] **prompt** is a sequence of characters used in a command-line interface to indicate readiness to accept commands.

The **console** is the CLI application itself and is a more general term to describe applications such as windows 'command prompt' and Unix' 'terminal'.



## A quick tour of PowerShell


Viewing the current directory and the path to it can be displayed using 'Get-Location'

```
Get-Location

> C:\Users\username
```

Notice that the path is printed out with backward-slashes ```\```. This is the conventional path format for Windows. In contrast, Linux follows a forward-slash ```/``` format for their paths.

Now we know where we are in the file system, but we need to learn what directories and files are in the current directory. To do this, we use ```Get-ChildItem```



```
Get-ChildItem

> Directory: C:\Users\username

> Mode                LastWriteTime         Length Name
> ----                -------------         ------ ----
> d-r---       10/10/2020     10:33                Desktop
> d-r---       10/10/2020     10:33                Documents
> d-r---       10/10/2020     10:33                Downloads
> d-r---       17/10/2020     15:10                example  
```

After executing ```Get-ChildItem```, PowerShell displays the directory it is assuming we want to see the content of, the current directory ```C:\Users\username```. Then all the directories and files are listed in a table format, giving some information
about the mode of the directories, last time they were changed, the length in bytes and the name.

Now we know where we are in the file system, we can navigate to another folder using ```Set-Location```. In this tour, we'll navigate into the directory called ```example```.

```
Set-Location example

Get-ChildItem

> Directory: C:\Users\username\example

> Mode                LastWriteTime         Length Name
> ----                -------------         ------ ----
```

Looks like this directory is empty. Let's make a directory called ```folders``` using ```mkdir``` (This is the exact same was as in Linux) and navigate into it.

```
mkdir folders


> Directory: C:\Users\username\example


> Mode                LastWriteTime         Length Name
> ----                -------------         ------ ----
> d-----       18/10/2020     12:20                folders

Set-Location folders

Get-Location

> C:\Users\username\example\folders
```

If you come from a Linux background or are interested in following the conventional Linux CLI commands as it is more universally understood, almost all of the Linux CLI commands you'll find described in the next section about Bash are aliased
to PowerShell commands. Therefore, you can type ```pwd``` and get the same effect as typing ```Get-Location```, PowerShell automatically translates it.


## A quick tour of Bash



To do things with a CLI we write commands. A CLI is very **similar to File Explorer** or other file browsers. We can navigate around the directories on the computer and perform tasks. ***Many act on the current directory you are in,*** though some run programs more generally.



We can view the current directory with the command **P**rint **W**orking **D**irectory ( ```pwd``` ).

```
pwd

> /Users/username/example/folders
```



We can **c**hange **d**irectory using ```cd``` followed by the path to where we want to go (separated from the command by a space).

```
cd ../../another_example/folder
pwd
> /Users/username/another_example/folder
```



A path is the names of the directories joined together with ```/``` characters. 

The ```../``` represents moving back one directory, so in the above examples we backed out of the directory called 'folders' and backed out of the directory 'example'. We also moved through the folder 'another_example' and into the directory inside that called folder.



We can view contents using the **l**i**s**t command ```ls```.

```
cd ../../
pwd

> username 

ls

> example another_example file.txt
```



When we moved back two directories and used the ```ls``` command, we see that in the username directory there is a text file called 'file.txt' and the two directories we've used so far called 'example' and 'another_example' respectively.



Inside directories we can run programs to perform actions. This might be to manage source code (such as git - which we will explore in other worksheets) or to perform actions on you system (such as changing file permissions). Most things that can be done on a computer can be done on the command line, and some things are much easier to do using a CLI.



## Running Commands

Both Window's PowerShell and Linux Bash run programs/applications in the same way on their respective CLI. To run a command, we type the command into the CLI where we are 'prompted' to write. Usually, a CLI will show a username followed by a symbol to indicate that you may type a command (often with a block indicating the location of your [caret](https://en.wikipedia.org/wiki/Caret_navigation)):

```
username $ ▒
```

For the operating system to execute the program, it must be added to the path variables. Path variables are just paths through the file structure to the location of the program on the computer. This informs the OS where the program
exists and what name executes it. Conveniently, this is done during the installation of most programs. Therefore, on PowerShell and Bash, we should just be able to type the name of the program to run it.

For example, we can run git by typing in git:

```
git
```

 As output, we would expect to see a manual output. Git as a program needs more information to do it's job than simply being run. We may see something like the following printed out:

```
> usage: git [--version] [--help] [-C <path>] [-c <name>=<value>]
           [--exec-path[=<path>]] [--html-path] [--man-path] [--info-path]
           [-p | --paginate | -P | --no-pager] [--no-replace-objects] [--bare]
           [--git-dir=<path>] [--work-tree=<path>] [--namespace=<name>]
           <command> [<args>]
```

If we received an error such as " ```sh: command not found: git``` " then we should either check for mistakes or ensure that the desired program is installed.



#### Commands With Arguments

Commands that take arguments are separated by a space. We initialise a git repository (as we will see later) by passing git a command of ```init```.

```
git init
```



In some cases, not only will we provide commands separated by spaces, but arguments too:



```
git add filename.txt code.cpp
```



In the example above, we pass the ```git add``` command two file names that git should track.



In general, you will be required to know the arguments required for commands on the command line, and so this may require reading the usage information. We can view the manual in the CLI for a command by using the man command and passing the name of the command we want information on:

```
man git
```

*** to exit the manual type ```:``` ```q``` ```enter```** 



You may also see 'flags' passed as arguments. These are typically single letter arguments that are used to specify additional behavior. For example, adding the ```-a``` flag to the ```ls``` command lists all directories, including hidden files.

```
ls -a
```



#### Writing Paths

We write paths to files as directory names separated by slashes. On Windows we use ```\``` and on Unix systems (Linux and OS X) we use ```/```.

***Remember to use the correct slash for your OS. We will be using Unix style ```/``` in these examples.**



We can provide either a ***relative*** path or an ***absolute*** path. 

A relative path starts from the current location (and is therefore relative to the current location of the shell) and uses ```../``` to step back out of the current directory. 



An absolute path gives the full path from the root of the system.  This would look something like the following on a Windows system for example starting on the ```C:``` drive.

```C:\Documents\ProgrammingInCPP\source.cpp```

> This provides the full path to a file called source.cpp



# Tips

#### Autocomplete

Most CLI will attempt to autocomplete what you are typing (particularly paths) when you press tab. You can use this to significantly speed up your navigation and use of the CLI.



#### Run previous command

Pressing ```up arrow``` in a CLI will often add the last run command at the prompt. Continuing to press the arrow key allows you to navigate through the history of commands that have been run - allowing you to quickly run a previous command again. *Useful for compiling code!*



# Resources

Command Reference:

[Excellent CLI command reference site](https://ss64.com/)

[power shell commands](https://ss64.com/ps/)

[bash commands](https://ss64.com/osx/)



**Videos for Windows:**

[Windows PowerShell [01] Introduction](https://www.youtube.com/watch?v=TUNNmVeyjW0)

[Learn and use PowerShell with just three commands](https://www.youtube.com/watch?v=FiTZgpRpWv0)



**Videos for OS X / Linux:**

[How to use the Command Line | Terminal Basics for Beginners](https://www.youtube.com/watch?v=5XgBd6rjuDQ)



