# Git Reference

This reference is a reminder of the commands for git __however__ always read the command. Often you will need to input information using the correct syntax. Urls will be dependent on your project and messages should be you own.

**Use this reference wisely...**

## Inspect a repo

##### View currently tracked files

Red files are untracked, green files are tracked.

```
git status
```



##### View changes

```
git diff
```



##### View commits

```
git log
```





## Manage a Repo

##### Git Clone

```
git clone https://gitlab.uwe.ac.uk/USER_NAME/PROJECT_NAME.git
```



##### Git global setup

> Typically only required for first use on a machine

```
git config --global user.name "User Name"
git config --global https://gitlab.uwe.ac.uk/USER_NAME/PROJECT_NAME.git
```



##### Create a new repository

This is the approach to create a new repo that you have already created online.

```
git clone https://gitlab.uwe.ac.uk/USER_NAME/PROJECT_NAME.git
cd example
touch README.md
git add README.md
git commit -m "added README"
git push -u origin master
```



##### Track files

Add all files by using ```.```

```
git add .
```

Add specific files by name

```
git add file.txt
```



##### Stop tracking a file (i.e. that you want to ignore)

```
git rm --cached file.txt
```

> For folders add the recursive flag ```git rm -r --cached dir_name```



##### Save changes to file

```
git commit -m "Your message here... I hope never read this in your history"
```



##### Pull changes from the remote repo

```
git pull
```



##### Push changes to the remote repo

```
git push
```



##### Push an existing folder

```
cd existing_folder
git init
git remote add origin https://gitlab.uwe.ac.uk/USER_NAME/PROJECT_NAME.git
git add .
git commit -m "Initial commit"
git push -u origin master
```



##### Push an existing Git repository

```
cd existing_repo
git remote rename origin old-origin
git remote add origin https://gitlab.uwe.ac.uk/USER_NAME/PROJECT_NAME.git
git push -u origin --all
git push -u origin --tags
```



## Other helpful things

##### Stash

Store the current changes but revert to the previous commit. Helpful if you loose track of something working or you want to non destructively try a different approach.

```
git stash
```

Bring back the stashed changes

```
git stash apply
```